/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2006, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 * \brief General Asterisk PBX capability definitions.
 * \par See also:
 *  \arg \ref AstVideoCaps
 *  \arg \ref Def_Channel
 *  \arg \ref channel_drivers
 *  \author John Martin, Aupix LTD
 */

/* \page AstVideoCaps Asterisk Video Capability Handling
 * \title Asterisk video capabilities
 *
 *	When setting up a video call, but also some audio calls, specifying the
 *	codec as the only capability is not enough. Therefore, we need to complement
 *	the codec negotiation with capabilities for each codec.
 *	For video, this is often format, framerate, bandwidth and variant.
 *	Some audio codecs have various bitrates and modes.
 *
 *	In order to sort this out, you need to read each RFC with a specification
 *	of the SDP payload format as well as the actual specification of the codec.
 *
 *	In order to handle this we need
 *	- Functions to parse configuration file settings for each codec
 *	- Function to compare settings during negotiation
 *	- Structures to store the settings
 *
 *	In this file, we specify attributes structures to handle a range of video
 *	codecs
 *
 *	- H.261
 *		- only handled in passthrough mode in Asterisk, no codec module
 *		- \ref format_h261.c
 *	- H.263
 *		- only handled in passthrough mode in Asterisk, no codec module
 *		- \ref format_h263.c
 *	- H.264
 *		- only handled in passthrough mode in Asterisk, no codec module
 *		- \ref format_h264.c
 *
 *	In protocols that use SDP, the Session Description Protocol, we negotiate
 * 	these attributes according to the SDP offer/answer model.
 */

#ifndef _ASTERISK_CAPABILITY_H
#define _ASTERISK_CAPABILITY_H

#include "asterisk/abstract_jb.h"

#include <unistd.h>
#ifdef POLLCOMPAT 
#include "asterisk/poll-compat.h"
#else
#include <sys/poll.h>
#endif

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#include "asterisk/compat.h"
#include "asterisk/frame.h"
#include "asterisk/config.h"
#include "asterisk/utils.h"
#include "asterisk/compiler.h"

/* Defines */
#define AST_VIDEO_SQCIF 	(0)
#define AST_VIDEO_QSIF 		(1)
#define AST_VIDEO_QCIF 		(2)
#define AST_VIDEO_QVGA 		(3)
#define AST_VIDEO_SIF 		(4)
#define AST_VIDEO_CIF 		(5)
#define AST_VIDEO_VGA 		(6)
#define AST_VIDEO_CIF4 		(7)
#define AST_VIDEO_SVGA 		(8)
#define AST_VIDEO_XGA 		(9)
#define AST_VIDEO_CIF16 	(10)

#define MAX_VIDEO_SIZES 11

#define AST_VIDEO_LEVEL_BASELINE (1 << 0)
#define AST_VIDEO_LEVEL_MAIN     (1 << 1)
#define AST_VIDEO_LEVEL_EXTENDED (1 << 2)

#define DEFAULT_MAX_CALL_BITRATE (256000)		/*!< Max bitrate for video */

/*! The h.264 profile level is the profile (eg 1.1 x 10) */
typedef enum
{
	level_1   = 10, /* 0x0A */
	level_1_1 = 11, /* 0x0B */
	level_1_2 = 12, /* 0x0C */
	level_1_3 = 13, /* 0x0D */
	level_2   = 20, /* 0x14 */
	level_2_1 = 21, /* 0x15 */
	level_2_2 = 22, /* 0x16 */
	level_3   = 30, /* 0x1E */
	level_3_1 = 31, /* 0x1F */
	level_3_2 = 32, /* 0x20 */
	level_4   = 40, /* 0x28 */
	level_4_1 = 41, /* 0x29 */
	level_4_2 = 42, /* 0x2A */
	level_5   = 50, /* 0x32 */
	level_5_1 = 51, /* 0x33 */
} ast_h264_profile_level;

/* MPEG 4 video profile_and_level_indication as in table G-1 ISO 14496-2 */

typedef enum
{
	sp_level_1 = 1, /* Simple profile level 1 */
	sp_level_2 = 2, /* Simple profile level 2 */
	sp_level_3 = 3, /* Simple profile level 3 */
	ssp_level_1 = 0x11, /* Simple Scalable profile level 1 */
	ssp_level_2 = 0x12, /* Simple Scalable profile level 2 */
	cp_level_1 = 0x21, /* core profile level 1 */
	cp_level_2 = 0x22, /* core profile level 2 */
	mp_level_2 = 0x32, /* main profile level 2 */
	mp_level_3 = 0x33, /* main profile level 3 */
	mp_level_4 = 0x34, /* main profile level 4 */
	bnp_level_2 = 0x42, /* N-bit profile level 2 */
	stp_level_1 = 0x51, /* scalable texture profile level 1 */
	sap_level_1 = 0x61, /* Simple face animation profile level 1*/
	sap_level_2 = 0x62, /* Simple face animation profile level 2*/
	fba_level_1 = 0x63, /* Simple FBA profile level 1 */
	fba_level_2 = 0x64, /* Simple FBA profile level 2 */
	batp_level_1 = 0x71, /* Basic Animated Texture Profile level 1 */
	batp_level_2 = 0x72, /* Basic Animated Texture Profile level 1 */
	hp_level_1 = 0x81,  /* Hybrid profile level 1 */
	asp_level_1 = 0x91, /* Advanced simple profile level 1 */
	asp_level_2 = 0x92, /* Advanced simple profile level 2 */
	asp_level_3 = 0x93, /* Advanced simple profile level 3 */
	asp_level_4 = 0x94, /* Advanced simple profile level 4 */
} ast_mpeg4v_profile_level;

/*!The following array defines the video sizes we know about */
struct videoSize {
	char*         type;
	unsigned int  num_mbs;
	unsigned char maxframerate;
}; 

extern struct videoSize videoSizes[];			/*! Declared in capability.c */
extern struct ast_capabilities ast_default_caps;	/*! Declared in capability.c */

/*! \brief ast_video_cap: Struct holding video capabilities for h.261, h.263*/
struct ast_h2613_video_cap {
	int           valid;                            /*!< 1 = This codec is specified, 0 = not specified */
	unsigned int  annexes;                          /*!< Video annexes - stored as bitfield 2^(annex-ascii(A))*/
        unsigned char profile;                          /*!< Video flags -- will only be used for h263+ */
        unsigned char level;                            /*!< Profile level -- will only be used for h263+ */
	unsigned char maxfr[MAX_VIDEO_SIZES];           /*!< Maximum framerate (framerate = 30/mpi)  */
	unsigned int  maxbr;                            /*!< Maximum supported bitrate for this capability */
};

/*! \brief ast_h264_video_cap: Struct holding video capabilities for h.264 */
struct ast_h264_video_cap {
	unsigned char valid;       /*!< 1 = This codec is specified, 0 = not specified */
	unsigned char rtpnum;      /*!< The rtp payload number used in this cap */
	unsigned char profile;     /*!< Video flags */
	unsigned char level;       /*!< Profile level */
	unsigned char constraint;  /*!< Constraint flags */
	unsigned int  maxmbps;     /*!< Maximum number of macroblocks per second */
	unsigned int  packet_mode; /*!< Packetisation mode */
	unsigned int  maxbr;       /*!< Maximum supported bitrate for this capability */
};

/*! \brief ast_capabilities: Struct holding all video capabilities */
struct ast_capabilities {
	unsigned int               cap;             /*!< Simple capability used to speed up search */
	unsigned char              prefs[32];       /*!< Audio preference order */
	unsigned int               maxcallbitrate;  /*!< Max allowed bitrate for this channel */
	unsigned int               maxvideobitrate; /*!< Max video bitrate */
	struct ast_h2613_video_cap h261;            /*!< H261 video capabilities */
	struct ast_h2613_video_cap h263;            /*!< H263 video capabilities */
	struct ast_h2613_video_cap h263p;           /*!< H263+ video capabilities */
	struct ast_h264_video_cap  mp4v;            /*!< MPEG4 video capabilities */
	struct ast_h264_video_cap  h264;            /*!< H264 video capabilities */
	unsigned char              t140;            /*!< T140 text capabilities */  
	unsigned char              red;             /*!< RED text capabilities */  
	struct ast_variable	   *x_settings;	    /*!< Settings outside of the specification received in SDP */
};

/*! \brief Set video capabilities of a channel
 * \note When videp caps need to be relayed to another part of call
 * \param chan channel to change the indication
 * \param capabilities
 * \return Returns 0 on success, -1 on failure
 */
int ast_set_capabilities(struct ast_channel *chan, struct ast_capabilities *chancaps);

struct ast_capabilities *ast_alloc_capabilities(void);

void ast_dump_caps(struct ast_capabilities *caps);

void ast_dump_h2613_video_cap(struct ast_h2613_video_cap *vidcap);

void ast_dump_h264_video_cap(struct ast_h264_video_cap *vidcap);

void ast_resolve_h2613_video_cap(struct ast_h2613_video_cap *dstcap, struct ast_h2613_video_cap *s1cap, struct ast_h2613_video_cap *s2cap);

void ast_resolve_h264_video_cap(struct ast_h264_video_cap *dstcap, struct ast_h264_video_cap *s1cap, struct ast_h264_video_cap *s2cap);

void ast_resolve_h263p_video_cap(struct ast_h2613_video_cap *dstcap, struct ast_h2613_video_cap *s1cap, struct ast_h2613_video_cap *s2cap);

void ast_resolve_mp4v_video_cap(struct ast_h264_video_cap *dstcap, struct ast_h264_video_cap *s1cap, struct ast_h264_video_cap *s2cap);

int ast_copy_capabilities(struct ast_capabilities *dstcap, struct ast_capabilities *srccap);

int ast_resolve_capabilities(struct ast_capabilities *dst, struct ast_capabilities *s1caps, struct ast_capabilities *s2caps, int debug);

int ast_are_caps_zero(struct ast_capabilities *caps);

int ast_set_capabilities_from_int(struct ast_capabilities *caps, int simplecap);

int ast_get_int_from_capabilities(struct ast_capabilities *caps);

int ast_get_negociated_caps(struct ast_channel *chan, struct ast_capabilities *chancaps);//Fusion

int check_set_video_bitrates(struct ast_capabilities *caps, int debug);

int parse_h2613_conf_line(const char *fmtstr, struct ast_h2613_video_cap *vidcap, unsigned int framerate);

int parse_mp4v_conf_line(const char *fmtstr, struct ast_h264_video_cap *vidcap, int framerate);

int parse_h264_conf_line(const char *fmtstr, struct ast_h264_video_cap *vidcap, int framerate);

#if 0 /* Need to move these functions from rtp to here... */
/*! \brief  Setting RTP payload types from lines in a SDP description: */
void ast_rtp_pt_clear(struct ast_rtp* rtp);

/*! \brief Set payload types to defaults */
void ast_rtp_pt_default(struct ast_rtp* rtp);

/*! \brief Copy payload types between RTP structures */
void ast_rtp_pt_copy(struct ast_rtp *dest, const struct ast_rtp *src);

void ast_rtp_set_m_type(struct ast_rtp* rtp, int pt);

void ast_rtp_set_rtpmap_type(struct ast_rtp* rtp, int pt,
			     char *mimeType, char *mimeSubtype,
			     enum ast_rtp_options options);

/*! \brief  Mapping between RTP payload format codes and Asterisk codes: */
struct rtpPayloadType ast_rtp_lookup_pt(struct ast_rtp* rtp, int pt);
int ast_rtp_lookup_code(struct ast_rtp* rtp, int isAstFormat, int code);

void ast_rtp_get_current_formats(struct ast_rtp* rtp,
			     int* astFormats, int* nonAstFormats);

/*! \brief  Mapping an Asterisk code into a MIME subtype (string): */
const char *ast_rtp_lookup_mime_subtype(int isAstFormat, int code,
					enum ast_rtp_options options);

/*! \brief Build a string of MIME subtype names from a capability list */
char *ast_rtp_lookup_mime_multiple(char *buf, size_t size, const int capability,
				   const int isAstFormat, enum ast_rtp_options options);

int ast_rtp_codec_setpref(struct ast_rtp *rtp, struct ast_codec_pref *prefs);

struct ast_codec_pref *ast_rtp_codec_getpref(struct ast_rtp *rtp);

int ast_rtp_codec_getformat(int pt);
#endif

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#endif /* _ASTERISK_CAPABILITY_H */
